/* eslint-disable no-case-declarations */

import { getKeyByPattern, jsonDataType } from "../utils";

/**
 * 通过 schema 判断当前 json 是否可以创建新的属性。
 * @param props
 * @param fieldInfo
 */
export var canFieldCreate = function canFieldCreate(data, fieldInfo) {
  var dataType = jsonDataType(data);
  var mergedValueSchema = fieldInfo.mergedValueSchema;
  if (!mergedValueSchema) return dataType === 'array' || dataType === 'object';
  var maxProperties = mergedValueSchema.maxProperties,
    properties = mergedValueSchema.properties,
    additionalProperties = mergedValueSchema.additionalProperties,
    patternProperties = mergedValueSchema.patternProperties,
    maxItems = mergedValueSchema.maxItems,
    items = mergedValueSchema.items,
    prefixItems = mergedValueSchema.prefixItems;
  var autoCompleteKeys = [];
  switch (dataType) {
    case 'object':
      /**
       * object可以创建新属性，需要关注的条件：
       * 1. patternProperties 不为空，我们默认 patternProperties 只要有可用的正则就肯定能再创建。
       * 2. additionalProperties 不为 false
       * 3. 不超过 maxLength
       */
      var nowKeys = Object.keys(data);
      // 1. 长度验证
      if (maxProperties !== undefined && nowKeys.length >= maxProperties) return false;
      // 收集 properties 中可以创建的+可自动补全的属性
      var restKeys = properties ? Object.keys(properties).filter(function (key) {
        return !nowKeys.includes(key);
      }) : [];
      // todo: 依据 dependencies 筛选可创建属性
      if (properties) autoCompleteKeys = autoCompleteKeys.concat(restKeys);
      // 2. additionalProperties 验证
      if (additionalProperties !== false) return autoCompleteKeys;
      // 3. patternProperties 有键
      if (patternProperties && Object.keys(patternProperties).length > 0) return autoCompleteKeys;
      // 4. 有无剩余键
      return restKeys.length > 0 ? autoCompleteKeys : false;
    case 'array':
      var prefixLength = prefixItems && items === false ? prefixItems.length : +Infinity;
      var maxLength = maxItems === undefined ? +Infinity : maxItems;
      return (maxLength < prefixLength ? data.length < maxLength : data.length < prefixLength) ? [] : false;
    default:
      return false;
  }
};

/**
 * 通过 schema 判断该字段(来自一个对象)是否可以重新命名
 * @param ctx
 * @param props
 * @param fieldInfo
 * @returns 返回字符串为不可命名(同时其也是字段名称)，返回正则为命名范围，返回空串即可命名
 */
export var canFieldRename = function canFieldRename(ctx, props, fieldInfo) {
  var field = props.field,
    fatherInfo = props.fatherInfo;
  var mergedEntrySchema = fieldInfo.mergedEntrySchema;
  // 注意，一个模式的 title 看 entryMap，如果有of等不理他
  var _ref = mergedEntrySchema || {},
    title = _ref.title;
  if (field === undefined) {
    return title ? title : ' ';
  }
  // 不是根节点，不保证 FatherInfo 一定存在，因为可能有抽屉！
  var _ref2 = fatherInfo !== null && fatherInfo !== void 0 ? fatherInfo : {},
    fatherValueEntry = _ref2.valueEntry,
    fatherType = _ref2.type;
  var fatherMergedValueSchema = ctx.getMergedSchema(fatherValueEntry);
  if (fatherType === 'array') {
    return title ? title + ' ' + field : field;
  } else if (!fatherMergedValueSchema) {
    return '';
  } else {
    var properties = fatherMergedValueSchema.properties,
      patternProperties = fatherMergedValueSchema.patternProperties;
    if (properties && properties[field]) return title ? title : field;
    var pattern = patternProperties ? getKeyByPattern(patternProperties, field) : undefined;
    if (pattern) return pattern;
    return '';
  }
};

/**
 * 通过 schema 判断该字段是否可删除。可删除条件：
 * 1. `field === undefined` 意味着根字段，不可删除
 * 2. 如果父亲是数组，只要不在数组 prefixItems 里面即可删除
 * 3. 如果父亲是对象，只要不在 required 里面即可删除
 *
 * @param ctx
 * @param props
 */
export var canFieldDelete = function canFieldDelete(ctx, props) {
  var fatherInfo = props.fatherInfo,
    field = props.field;
  if (field === undefined) return false;
  if (fatherInfo) {
    var fatherValueEntry = fatherInfo.valueEntry;
    switch (fatherInfo.type) {
      case 'array':
        var _ref3 = ctx.getMergedSchema(fatherValueEntry) || {},
          prefixItems = _ref3.prefixItems;
        var index = parseInt(field);
        return prefixItems ? index >= prefixItems.length : true;
      case 'object':
        var fatherMergedValueSchema = ctx.getMergedSchema(fatherValueEntry);
        if (fatherMergedValueSchema && fatherMergedValueSchema.required) {
          return !fatherMergedValueSchema.required.includes(field);
        } else {
          return true;
        }
      default:
        console.error('意外的判断情况');
        return false;
    }
  }
  return false;
};

/**
 * 字段是否为 required 字段
 * @param field
 * @param fatherInfo
 * @returns
 */
export var isFieldRequired = function isFieldRequired(field, fatherInfo) {
  if (!fatherInfo || !field) return false;
  var required = fatherInfo.required;
  return required instanceof Array && required.indexOf(field) > -1;
};